/*
** mts.c -- definitions for the mail transport system
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>   /* for snprintf() */
#include <h/nmh.h>
#include <h/utils.h>
#include <ctype.h>
#include <stdio.h>
#include <h/mts.h>
#include <pwd.h>
#include <netdb.h>

#ifdef HAVE_SYS_UTSNAME_H
# include <sys/utsname.h>
#endif

#define NOTOK  (-1)
#define OK     0

/*
** static prototypes
*/
static void getuserinfo(void);

/* Cache the username and fullname of the user */
static char username[BUFSIZ];
static char fullname[BUFSIZ];


/*
** Get the fully qualified name of the local host.
*/
char *
LocalName(void)
{
	static char buffer[BUFSIZ] = "";
	struct addrinfo hints, *res;
#ifdef HAVE_UNAME
	struct utsname name;
#endif

	/* check if we have cached the local name */
	if (buffer[0])
		return buffer;

	memset(buffer, 0, sizeof(buffer));
#ifdef HAVE_UNAME
	/* first get our local name */
	uname(&name);
	strncpy(buffer, name.nodename, sizeof(buffer) - 1);
#else
	/* first get our local name */
	gethostname(buffer, sizeof(buffer) - 1);
#endif
	/* now fully qualify our name */

	memset(&hints, 0, sizeof(hints));
	hints.ai_flags = AI_CANONNAME;
	hints.ai_family = PF_UNSPEC;
	if (getaddrinfo(buffer, NULL, &hints, &res) == 0) {
		strncpy(buffer, res->ai_canonname, sizeof(buffer) - 1);
		freeaddrinfo(res);
	}

	return buffer;
}


/*
** This is only for UUCP mail.  It gets the hostname
** as part of the UUCP "domain".
*/
char *
SystemName(void)
{
	static char buffer[BUFSIZ] = "";

#ifdef HAVE_UNAME
	struct utsname name;
#endif

	/* check if we have cached the system name */
	if (buffer[0])
		return buffer;

#ifdef HAVE_UNAME
	uname(&name);
	strncpy(buffer, name.nodename, sizeof(buffer));
#else
	gethostname(buffer, sizeof(buffer));
#endif

	return buffer;
}


/*
** Get the username of current user
*/
char *
getusername(void)
{
	if (username[0] == '\0')
		getuserinfo();

	return username;
}


/*
** Get full name of current user (typically from GECOS
** field of password file).
*/
char *
getfullname(void)
{
	if (username[0] == '\0')
		getuserinfo();

	return fullname;
}


/*
** Find the user's username and full name, and cache them.
*/
static void
getuserinfo(void)
{
	register unsigned char *cp;
	register char *np;
	register struct passwd *pw;

	if ((pw = getpwuid(getuid())) == NULL
		|| pw->pw_name == NULL
		|| *pw->pw_name == '\0') {
		strncpy(username, "unknown", sizeof(username));
		snprintf(fullname, sizeof(fullname), "The Unknown User-ID (%d)",
			(int) getuid());
		return;
	}

	np = pw->pw_gecos;

	/*
	** Get the user's real name from the GECOS field.  Stop once
	** we hit a ',', which some OSes use to separate other 'finger'
	** information in the GECOS field, like phone number.
	*/
	for (cp = fullname; *np != '\0' && *np != ',';) {
#ifndef BSD42
		*cp++ = *np++;
#else /* BSD42 */
		/*
		** On BSD(-derived) systems, the system utilities that
		** deal with the GECOS field (finger, mail, sendmail,
		** etc.) translate any '&' character in it to the login name,
		** with the first letter capitalized.  So, for instance,
		** fingering a user "bob" with the GECOS field "& Jones"
		** would reveal him to be "In real life: Bob Jones".
		** Surprisingly, though, the OS doesn't do the translation
		** for you, so we have to do it manually here.
		*/
		if (*np == '&') {  /* blech! */
			strcpy(cp, pw->pw_name);
			*cp = toupper(*cp);
			while (*cp)
				cp++;
			np++;
		} else {
			*cp++ = *np++;
		}
#endif /* BSD42 */
	}
	*cp = '\0';
	strncpy(username, pw->pw_name, sizeof(username));

	/*
	** The $SIGNATURE environment variable overrides the GECOS field's
	** idea of your real name.
	*/
	if ((cp = getenv("SIGNATURE")) && *cp)
		strncpy(fullname, cp, sizeof(fullname));

	if (strchr(fullname, '.')) {  /*  quote any .'s */
		char tmp[BUFSIZ];

		/* should quote "'s too */
		snprintf(tmp, sizeof(tmp), "\"%s\"", fullname);
		strncpy(fullname, tmp, sizeof(fullname));
	}

	return;
}
